@@ -704,6 +704,38 @@ rsvg_characters_impl (RsvgHandle * ctx, const xmlChar * ch, int len)
     if (!ch || !len)
         return;
 
+	if (ctx->priv->currentnode)
+		{
+			if (!strcmp ("tspan", ctx->priv->currentnode->type->str) ||
+				!strcmp ("text", ctx->priv->currentnode->type->str))
+				{
+					guint i;
+
+					/* find the last CHARS node in the text or tspan node, so that we
+					   can coalesce the text, and thus avoid screwing up the Pango layouts */
+					self = NULL;
+					for (i = 0; i < ctx->priv->currentnode->children->len; i++) {
+						RsvgNode *node = g_ptr_array_index (ctx->priv->currentnode->children, i);
+						if (!strcmp (node->type->str, "RSVG_NODE_CHARS")) {
+							self = (RsvgNodeChars*)node;
+						}
+					}
+
+					if (self != NULL) {
+						if (!g_utf8_validate ((char *) ch, len, NULL)) {
+							char *utf8;
+							utf8 = rsvg_make_valid_utf8 ((char *) ch, len);
+							g_string_append (self->contents, utf8);
+							g_free (utf8);
+						} else {
+							g_string_append_len (self->contents, (char *)ch, len);
+						}
+
+						return;
+					}
+				}
+		}
+
     self = g_new (RsvgNodeChars, 1);
     _rsvg_node_init (&self->super);
 
@@ -1082,9 +1114,8 @@ rsvg_handle_close_impl (RsvgHandle * handle, GError ** error)
 static void
 rsvg_state_free_func (gpointer data, gpointer user_data)
 {
-    RsvgDrawingCtx *ctx = (RsvgDrawingCtx *) user_data;
     rsvg_state_finalize ((RsvgState *) data);
-    g_mem_chunk_free (ctx->state_allocator, data);
+    g_slice_free (RsvgState, data);
 }
 
 void
@@ -1101,8 +1132,6 @@ rsvg_drawing_ctx_free (RsvgDrawingCtx * handle)
     if (handle->base_uri)
         g_free (handle->base_uri);
 
-    g_mem_chunk_destroy (handle->state_allocator);
-
     if (handle->pango_context != NULL)
         g_object_unref (handle->pango_context);
 
@@ -1281,8 +1310,6 @@ _rsvg_find_bbox (RsvgHandle * handle)
 
     ctx->state = NULL;
 
-    ctx->state_allocator = g_mem_chunk_create (RsvgState, 256, G_ALLOC_AND_FREE);
-
     ctx->defs = handle->priv->defs;
     ctx->base_uri = g_strdup (handle->priv->base_uri);
     ctx->dpi_x = handle->priv->dpi_x;
@@ -1598,10 +1625,6 @@ rsvg_init (void)
 #endif
 
     xmlInitParser ();
-
-#ifdef HAVE_GNOME_VFS
-    gnome_vfs_init ();
-#endif
 }
 
 /**
